import numpy as np


def hc_obj(x, param):
    """Implements the nonlinear hanging chain objective.

    Note that only intermediate mass points have degree of freedom.

    We assume that x = [y1, z1, y2, z2 ..., yN, zN].T
    """

    N = np.size(x, 0) // 2
    y = x[0:2*N:2]
    z = x[1:2*N:2]

    L   = param['L']
    D   = param['D']
    m   = param['m']
    g   = param['g']
    LoN = L / (N+1)  # (N+2-1) since total number of masses is N+2

    xi = param['xi']
    xf = param['xf']

    # Initialize the potential
    f = 0
    # Iterate on the points and update the potential
    for k in range(N-1):
        f = f + 0.5*D*(np.sqrt(
                (y[k+1] - y[k])**2 + (z[k+1] - z[k])**2
            ) - LoN)**2 + m*g*z[k]
    # Add the last gravitational energy
    f = f + z[N-1]*m*g

    # Adding potential of fixed points (we have an unconstrained problem)
    f = f + 0.5*D*(np.sqrt(
            (xi[0] - y[0])**2 + (xi[1] - z[0])**2
        ) - LoN)**2
    f = f + 0.5*D*(np.sqrt(
            (xf[0] - y[N-1])**2 + (xf[1] - z[N-1])**2
        ) - LoN)**2

    return f
